@@ -120,8 +120,12 @@ cString cSatipDevice::GetSatipStatus(void)
             info = cString::sprintf("%sCardIndex: %d  HasLock: yes  Strength: %d  Quality: %d%s\n", *info, device->CardIndex(), device->SignalStrength(), device->SignalQuality(), live ? "  Live: yes" : "");
          else
             info = cString::sprintf("%sCardIndex: %d  HasLock: no\n", *info, device->CardIndex());
-         if (channel && channel->Number() > 0)
-            info = cString::sprintf("%sTransponder: %d  Channel: %s\n", *info, (channel && channel->Number() > 0) ? channel->Transponder() : 0, (channel && channel->Number() > 0) ? channel->Name() : "---");
+         if (channel) {
+            if (channel->Number() > 0 && device->Receiving())
+               info = cString::sprintf("%sTransponder: %d  Channel: %s\n", *info, channel->Transponder(), channel->Name());
+            else
+               info = cString::sprintf("%sTransponder: %d\n", *info, channel->Transponder());
+            }
          if (timers)
             info = cString::sprintf("%sRecording: %d timer%s\n", *info, timers, (timers > 1) ? "s" : "");
          info = cString::sprintf("%s\n", *info);
@@ -136,7 +140,7 @@ cString cSatipDevice::GetGeneralInformation(void)
 #if defined(APIVERSNUM) && APIVERSNUM >= 20301
   LOCK_CHANNELS_READ;
 #endif
-  return cString::sprintf("SAT>IP device: %d\nCardIndex: %d\nStream: %s\nSignal: %s\nStream bitrate: %s\n%sChannel: %s",
+  return cString::sprintf("SAT>IP device: %d\nCardIndex: %d\nStream: %s\nSignal: %s\nStream bitrate: %s\n%sChannel: %s\n",
                           deviceIndexM, CardIndex(),
                           pTunerM ? *pTunerM->GetInformation() : "",
                           pTunerM ? *pTunerM->GetSignalStatus() : "",
@@ -206,6 +210,8 @@ cString cSatipDevice::DeviceType(void) const
 cString cSatipDevice::DeviceName(void) const
 {
   debug16("%s [device %u]", __PRETTY_FUNCTION__, deviceIndexM);
+  if (!Receiving())
+     return cString::sprintf("%s %d", *DeviceType(), deviceIndexM);
   return deviceNameM;
 }
 
@@ -355,6 +361,7 @@ bool cSatipDevice::SetChannelDevice(const cChannel *channelP, bool liveViewP)
      }
   else if (pTunerM) {
      pTunerM->SetSource(NULL, 0, NULL, deviceIndexM);
+     deviceNameM = cString::sprintf("%s %d", *DeviceType(), deviceIndexM);
      return true;
      }
   return false;
@@ -366,7 +373,7 @@ bool cSatipDevice::SetPid(cPidHandle *handleP, int typeP, bool onP)
   if (pTunerM && handleP && handleP->pid >= 0) {
      if (onP)
         return pTunerM->SetPid(handleP->pid, typeP, true);
-     else if (!handleP->used)
+     else if (!handleP->used && pSectionFilterHandlerM && !pSectionFilterHandlerM->Exists(handleP->pid))
         return pTunerM->SetPid(handleP->pid, typeP, false);
      }
   return true;
@@ -472,6 +479,13 @@ int cSatipDevice::GetCISlot(void)
   return slot;
 }
 
+cString cSatipDevice::GetTnrParameterString(void)
+{
+   if (channelM.Ca())
+      return GetTnrUrlParameters(&channelM);
+   return NULL;
+}
+
 bool cSatipDevice::IsIdle(void)
 {
   return !Receiving();
