@@ -27,7 +27,7 @@
 #define GITVERSION ""
 #endif
 
-       const char VERSION[]     = "2.2.3" GITVERSION;
+       const char VERSION[]     = "2.2.4" GITVERSION;
 static const char DESCRIPTION[] = trNOOP("SAT>IP Devices");
 
 class cPluginSatip : public cPlugin {
@@ -35,6 +35,7 @@ private:
   unsigned int deviceCountM;
   cSatipDiscoverServers *serversM;
   void ParseServer(const char *paramP);
+  void ParsePortRange(const char *paramP);
   int ParseCicams(const char *valueP, int *cicamsP);
   int ParseSources(const char *valueP, int *sourcesP);
   int ParseFilters(const char *valueP, int *filtersP);
@@ -83,11 +84,13 @@ const char *cPluginSatip::CommandLineHelp(void)
   // Return a string that describes all known command line options.
   return "  -d <num>, --devices=<number>  set number of devices to be created\n"
          "  -t <mode>, --trace=<mode>     set the tracing mode\n"
-         "  -s <ipaddr>|<model>|<desc>, --server=<ipaddr1>|<model1>|<desc1>;<ipaddr2>|<model2>|<desc2>\n"
+         "  -s <ipaddr>|<model>|<desc>, --server=<ipaddr1>|<model1>|<desc1>;<ipaddr2>:<port>|<model2>:<filter>|<desc2>:<quirk>\n"
          "                                define hard-coded SAT>IP server(s)\n"
          "  -D, --detach                  set the detached mode on\n"
          "  -S, --single                  set the single model server mode on\n"
-         "  -n, --noquirks                disable all the server quirks\n";
+         "  -n, --noquirks                disable autodetection of the server quirks\n"
+         "  -p, --portrange=<start>-<end> set a range of ports used for the RT[C]P server\n"
+         "                                a minimum of 2 ports per device is required.\n";
 }
 
 bool cPluginSatip::ProcessArgs(int argc, char *argv[])
@@ -98,6 +101,7 @@ bool cPluginSatip::ProcessArgs(int argc, char *argv[])
     { "devices",  required_argument, NULL, 'd' },
     { "trace",    required_argument, NULL, 't' },
     { "server",   required_argument, NULL, 's' },
+    { "portrange",required_argument, NULL, 'p' },
     { "detach",   no_argument,       NULL, 'D' },
     { "single",   no_argument,       NULL, 'S' },
     { "noquirks", no_argument,       NULL, 'n' },
@@ -105,8 +109,9 @@ bool cPluginSatip::ProcessArgs(int argc, char *argv[])
     };
 
   cString server;
+  cString portrange;
   int c;
-  while ((c = getopt_long(argc, argv, "d:t:s:DSn", long_options, NULL)) != -1) {
+  while ((c = getopt_long(argc, argv, "d:t:s:p:DSn", long_options, NULL)) != -1) {
     switch (c) {
       case 'd':
            deviceCountM = strtol(optarg, NULL, 0);
@@ -126,10 +131,15 @@ bool cPluginSatip::ProcessArgs(int argc, char *argv[])
       case 'n':
            SatipConfig.SetDisableServerQuirks(true);
            break;
+      case 'p':
+           portrange = optarg;
+           break;
       default:
            return false;
       }
     }
+  if (!isempty(*portrange))
+     ParsePortRange(portrange);
   // this must be done after all parameters are parsed
   if (!isempty(*server))
      ParseServer(*server);
@@ -222,7 +232,9 @@ void cPluginSatip::ParseServer(const char *paramP)
   while (r) {
         r = skipspace(r);
         debug3("%s server[%d]=%s", __PRETTY_FUNCTION__, n, r);
-        cString serverAddr, serverModel, serverDescription;
+        cString serverAddr, serverModel, serverFilters, serverDescription;
+        int serverQuirk = cSatipServer::eSatipQuirkNone;
+        int serverPort = SATIP_DEFAULT_RTSP_PORT;
         int n2 = 0;
         char *s2, *p2 = r;
         char *r2 = strtok_r(p2, "|", &s2);
@@ -230,13 +242,34 @@ void cPluginSatip::ParseServer(const char *paramP)
               debug3("%s param[%d]=%s", __PRETTY_FUNCTION__, n2, r2);
               switch (n2++) {
                      case 0:
+                          {
                           serverAddr = r2;
+                          char *r3 = strchr(r2, ':');
+                          if (r3) {
+                             serverPort = strtol(r3 + 1, NULL, 0);
+                             serverAddr = serverAddr.Truncate(r3 - r2);
+                             }
+                          }
                           break;
                      case 1:
+                          {
                           serverModel = r2;
+                          char *r3 = strchr(r2, ':');
+                          if (r3) {
+                             serverFilters = r3 + 1;
+                             serverModel = serverModel.Truncate(r3 - r2);
+                             }
+                          }
                           break;
                      case 2:
+                          {
                           serverDescription = r2;
+                          char *r3 = strchr(r2, ':');
+                          if (r3) {
+                             serverQuirk = strtol(r3 + 1, NULL, 0);
+                             serverDescription = serverDescription.Truncate(r3 - r2);
+                             }
+                          }
                           break;
                      default:
                           break;
@@ -244,10 +277,10 @@ void cPluginSatip::ParseServer(const char *paramP)
               r2 = strtok_r(NULL, "|", &s2);
               }
         if (*serverAddr && *serverModel && *serverDescription) {
-           debug1("%s ipaddr=%s model=%s desc=%s", __PRETTY_FUNCTION__, *serverAddr, *serverModel, *serverDescription);
+           debug1("%s ipaddr=%s port=%d model=%s (%s) desc=%s (%d)", __PRETTY_FUNCTION__, *serverAddr, serverPort, *serverModel, *serverFilters, *serverDescription, serverQuirk);
            if (!serversM)
               serversM = new cSatipDiscoverServers();
-           serversM->Add(new cSatipDiscoverServer(*serverAddr, *serverModel, *serverDescription));
+           serversM->Add(new cSatipDiscoverServer(*serverAddr, serverPort, *serverModel, *serverFilters, *serverDescription, serverQuirk));
            }
         ++n;
         r = strtok_r(NULL, ";", &s);
@@ -255,6 +288,37 @@ void cPluginSatip::ParseServer(const char *paramP)
   FREE_POINTER(p);
 }
 
+void cPluginSatip::ParsePortRange(const char *paramP)
+{
+  char *s, *p = skipspace(paramP);
+  char *r = strtok_r(p, "-", &s);
+  unsigned int rangeStart = 0;
+  unsigned int rangeStop = 0;
+  if (r) {
+     rangeStart = strtol(r, NULL, 0);
+     r = strtok_r(NULL, "-", &s);
+     }
+  if (r)
+     rangeStop = strtol(r, NULL, 0);
+  else {
+     error("Port range argument not valid '%s'", paramP);
+     rangeStart = 0;
+     rangeStop = 0;
+     }
+  if (rangeStart % 2) {
+     error("The given range start port must be even!");
+     rangeStart = 0;
+     rangeStop = 0;
+     }
+  else if (rangeStop - rangeStart + 1 < deviceCountM * 2) {
+     error("The given port range is to small: %d < %d!", rangeStop - rangeStart + 1, deviceCountM * 2);
+     rangeStart = 0;
+     rangeStop = 0;
+     }
+  SatipConfig.SetPortRangeStart(rangeStart);
+  SatipConfig.SetPortRangeStop(rangeStop);
+}
+
 int cPluginSatip::ParseCicams(const char *valueP, int *cicamsP)
 {
   debug1("%s (%s,)", __PRETTY_FUNCTION__, valueP);
@@ -342,6 +406,8 @@ bool cPluginSatip::SetupParse(const char *nameP, const char *valueP)
      for (unsigned int i = 0; i < DisabledFiltersCount; ++i)
          SatipConfig.SetDisabledFilters(i, DisabledFilters[i]);
      }
+  else if (!strcasecmp(nameP, "TransportMode"))
+     SatipConfig.SetTransportMode(atoi(valueP));
   else
      return false;
   return true;
